// Stage3D Shoot-em-up Tutorial Part2
// by Christer Kaitila - www.mcfunkypants.com
// Created for active.tutsplus.com

package 
{
	[SWF(width = "600", height = "400", frameRate = "60", backgroundColor = "#000000")]

	import flash.display3D.*;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageQuality;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.ErrorEvent;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;
	import flash.utils.getTimer;
		
	public class Main extends Sprite 
	{
		// the keyboard control system
		private var _controls : GameControls;
		// don't update the menu too fast
		private var nothingPressedLastFrame:Boolean = false;
		// timestamp of the current frame
		public var currentTime: int;
		// player one's entity
		public var thePlayer:Entity;
		// main menu = 0 or current level number
		private var _state : int = 0;
		// the title screen batch
		private var _mainmenu : GameMenu;
		// the sound system
		private var _sfx : GameSound;	
		// the background stars
		private var _bg : GameBackground;	
		
		private var _entities : EntityManager;
		private var _spriteStage : LiteSpriteStage;
		private var _gui : GameGUI;
		private var _width : Number = 600;
		private var _height : Number = 400;
		public var context3D : Context3D;
		
		// constructor function for our game
		public function Main():void 
		{
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		// called once flash is ready
		private function init(e:Event = null):void 
		{
			_controls = new GameControls(stage);
			removeEventListener(Event.ADDED_TO_STAGE, init);
			stage.quality = StageQuality.LOW;
			stage.align = StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.addEventListener(Event.RESIZE, onResizeEvent);
			trace("Init Stage3D...");
			_gui = new GameGUI("Stage3D Shoot-em-up Tutorial Part 2");
			addChild(_gui);
			stage.stage3Ds[0].addEventListener(Event.CONTEXT3D_CREATE, onContext3DCreate);
			stage.stage3Ds[0].addEventListener(ErrorEvent.ERROR, errorHandler);
			stage.stage3Ds[0].requestContext3D(Context3DRenderMode.AUTO);
			trace("Stage3D requested...");		
			
			_sfx = new GameSound();
		}
				
		// this is called when the 3d card has been set up
		// and is ready for rendering using stage3d
		private function onContext3DCreate(e:Event):void 
		{
			trace("Stage3D context created! Init sprite engine...");
			context3D = stage.stage3Ds[0].context3D;
			initSpriteEngine();
		}
		
		// this can be called when using an old version of flash
		// or if the html does not include wmode=direct
		private function errorHandler(e:ErrorEvent):void 
		{
			trace("Error while setting up Stage3D: "+e.errorID+" - " +e.text);
		}

		protected function onResizeEvent(event:Event) : void
		{
			trace("resize event...");
			
			// Set correct dimensions if we resize
			_width = stage.stageWidth;
			_height = stage.stageHeight;
			
			// Resize Stage3D to continue to fit screen
			var view:Rectangle = new Rectangle(0, 0, _width, _height);
			if ( _spriteStage != null ) {
				_spriteStage.position = view;
			}
			if(_entities != null) {
				_entities.setPosition(view);
			}
			if(_mainmenu != null) {
				_mainmenu.setPosition(view);
			}
		}
		
		private function initSpriteEngine():void 
		{
			// init a gpu sprite system
			var stageRect:Rectangle = new Rectangle(0, 0, _width, _height); 
			_spriteStage = new LiteSpriteStage(stage.stage3Ds[0], context3D, stageRect);
			_spriteStage.configureBackBuffer(_width,_height);
			
			// create the background stars
			_bg = new GameBackground(stageRect);
			_bg.createBatch(context3D);
			_spriteStage.addBatch(_bg.batch);
			_bg.initBackground();
			
			// create a single rendering batch
			// which will draw all sprites in one pass
			var view:Rectangle = new Rectangle(0,0,_width,_height)
			_entities = new EntityManager(stageRect);
			_entities.createBatch(context3D);
			_spriteStage.addBatch(_entities.batch);
			
			// create the logo/titlescreen main menu
			_mainmenu = new GameMenu(stageRect);
			_mainmenu.createBatch(context3D);
			_spriteStage.addBatch(_mainmenu.batch);
			
			// tell the gui where to grab statistics from
			_gui.statsTarget = _entities; 
			
			// start the render loop
			stage.addEventListener(Event.ENTER_FRAME,onEnterFrame);

			// only used for the menu
			stage.addEventListener(MouseEvent.MOUSE_DOWN, mouseDown);   
			stage.addEventListener(MouseEvent.MOUSE_MOVE, mouseMove); 
		}
		
		public function playerLogic(me:Entity):void
		{
			me.speedY = me.speedX = 0;
			if (_controls.pressing.up)
				me.speedY = -4;
			if (_controls.pressing.down)
				me.speedY = 4;
			if (_controls.pressing.left)
				me.speedX = -4;
			if (_controls.pressing.right)
				me.speedX = 4;
				
			// keep on screen
			if (me.sprite.position.x < 0)
				me.sprite.position.x = 0;
			if (me.sprite.position.x > _width)
				me.sprite.position.x = _width;
			if (me.sprite.position.y < 0)
				me.sprite.position.y = 0;
			if (me.sprite.position.y > _height)
				me.sprite.position.y = _height;
		}

		private function mouseDown(e:MouseEvent):void   
		{   
			trace('mouseDown at '+e.stageX+','+e.stageY);
			if (_state == 0) // are we at the main menu?
			{
				if (_mainmenu && _mainmenu.activateCurrentMenuItem(getTimer()))
				{ // if the above returns true we should start the game
					startGame();
				}
			}
		}

		private function mouseMove(e:MouseEvent):void   
		{
			if (_state == 0) // are we at the main menu?
			{
				// select menu items via mouse
				if (_mainmenu) _mainmenu.mouseHighlight(e.stageX, e.stageY);
			}
		}

		// handle any player input
		private function processInput():void
		{
			if (_state == 0) // are we at the main menu?
			{
				// select menu items via keyboard
				if (_controls.pressing.down || _controls.pressing.right)
				{
					if (nothingPressedLastFrame) 
					{
						_sfx.playGun(1);
						_mainmenu.nextMenuItem();
						nothingPressedLastFrame = false;
					}
				}
				else if (_controls.pressing.up || _controls.pressing.left)
				{
					if (nothingPressedLastFrame) 
					{
						_sfx.playGun(1);
						_mainmenu.prevMenuItem();
						nothingPressedLastFrame = false;
					}
				}
				else if (_controls.pressing.fire)
				{
					if (_mainmenu.activateCurrentMenuItem(getTimer()))
					{ // if the above returns true we should start the game
						startGame();
					}
				}
				else
				{
					// this ensures the menu doesn't change too fast
					nothingPressedLastFrame = true;
				}
			}
			else 
			{
				// we are NOT at the main menu: we are actually playing the game
				// in future versions we will add projectile
				// spawning functinality here to fire bullets
				if (_controls.pressing.fire)
				{
					_sfx.playGun(1);
					_entities.shootBullet();
				}
			}
		}
		
		private function startGame():void
		{
			trace("Starting game!");
			_state = 1;
			_spriteStage.removeBatch(_mainmenu.batch);
			_sfx.playMusic();
			// add the player entity to the game!
			thePlayer = _entities.addPlayer(playerLogic);			
		}
		
		// this function draws the scene every frame
		private function onEnterFrame(e:Event):void 
		{
			try 
			{
				// grab timestamp of current frame
				currentTime = getTimer();

				// erase the previous frame
				context3D.clear(0, 0, 0, 1);
				
				// for debugging the input manager, update the gui
				_gui.titleText = _controls.textDescription();
				
				// process any player input
				processInput();

				// scroll the background
				_bg.update(currentTime);
				
				// update the main menu titlescreen
				if (_state == 0)
					_mainmenu.update(currentTime);
				
				// keep adding more sprites - FOREVER!
				// this is a test of the entity manager's
				// object reuse "pool"
				_entities.addEntity();
				
				// move/animate all entities
				_entities.update(currentTime);
				
				// draw all entities
				_spriteStage.render();

				// update the screen
				context3D.present();
			}
			catch (e:Error) 
			{
				// this can happen if the computer goes to sleep and
				// then re-awakens, requiring reinitialization of stage3D
				// (the onContext3DCreate will fire again)
			}
		}
	} // end class
} // end package